/**
 * Copyright (c) 2000-present Liferay, Inc. All rights reserved.
 *
 * The contents of this file are subject to the terms of the Liferay Enterprise
 * Subscription License ("License"). You may not use this file except in
 * compliance with the License. You can obtain a copy of the License by
 * contacting Liferay, Inc. See the License for the specific language governing
 * permissions and limitations under the License, including but not limited to
 * distribution rights of the Software.
 *
 *
 *
 */

import {fetch, objectToFormData, openSelectionModal} from 'frontend-js-web';

export default function propsTransformer({
	additionalProps: {eventName, getItemTypeURL, itemSelectorURL, modalTitle},
	portletNamespace,
	...props
}) {
	const url = new URL(itemSelectorURL);

	return {
		...props,
		onClick() {
			const classNameIdInput = document.getElementById(
				`${portletNamespace}classNameId`
			);

			const classPKInput = document.getElementById(
				`${portletNamespace}classPK`
			);

			const classTypeIdInput = document.getElementById(
				`${portletNamespace}classTypeId`
			);

			const itemSubtypeElement = document.getElementById(
				`${portletNamespace}itemSubtype`
			);

			const itemSubtypeLabel = document.getElementById(
				`${portletNamespace}itemSubtypeLabel`
			);

			const itemTypeLabel = document.getElementById(
				`${portletNamespace}itemTypeLabel`
			);

			const originalTitleInput = document.getElementById(
				`${portletNamespace}originalTitle`
			);

			const titleInput = document.getElementById(
				`${portletNamespace}title`
			);

			const typeInput = document.getElementById(
				`${portletNamespace}type`
			);

			openSelectionModal({
				onSelect: (selectedItem) => {
					if (selectedItem) {
						let infoItem = {
							...selectedItem,
						};

						let value;

						if (typeof selectedItem.value === 'string') {
							try {
								value = JSON.parse(selectedItem.value);
							}
							catch (error) {}
						}
						else if (
							selectedItem.value &&
							typeof selectedItem.value === 'object'
						) {
							value = selectedItem.value;
						}

						if (value) {
							delete infoItem.value;
							infoItem = {...value};
						}

						classNameIdInput.value = infoItem.classNameId;
						classPKInput.value = infoItem.classPK;
						classTypeIdInput.value = infoItem.classTypeId || 0;
						originalTitleInput.value = infoItem.title || '';
						titleInput.value = infoItem.title || '';
						typeInput.value = infoItem.type || '';

						const namespacedInfoItem = Liferay.Util.ns(
							portletNamespace,
							infoItem
						);

						fetch(getItemTypeURL, {
							body: objectToFormData(namespacedInfoItem),
							method: 'POST',
						})
							.then((response) => response.json())
							.then((jsonResponse) => {
								itemTypeLabel.innerHTML =
									jsonResponse.itemType || '';
								itemSubtypeLabel.innerHTML =
									jsonResponse.itemSubtype || '';

								itemSubtypeElement.classList.toggle(
									'd-none',
									!jsonResponse.itemSubtype
								);
							})
							.catch(() => {});
					}
				},
				selectEventName: eventName,
				title: modalTitle,
				url: url.href,
			});
		},
	};
}
